#include <msp/core/refptr.h>
#include <msp/fs/utils.h>
#include <msp/io/file.h>
#include <msp/io/memory.h>
#include "image.h"
#include "imageloader.h"

using namespace std;

namespace Msp {
namespace Graphics {

Image::Data::Data():
	fmt(RGB),
	width(0),
	height(0),
	data(0)
{ }

Image::Data::Data(const Data &other):
	fmt(other.fmt),
	width(other.width),
	height(other.height),
	stride(other.stride),
	data(other.data ? new char[stride*height] : 0)
{
	if(data)
		copy(other.data, other.data+stride*height, data);
}

Image::Data &Image::Data::operator=(const Data &other)
{
	delete[] data;
	data = 0;

	fmt = other.fmt;
	width = other.width;
	height = other.height;
	stride = other.stride;

	if(other.data)
	{
		data = new char[stride*height];
		copy(other.data, other.data+stride*height, data);
	}

	return *this;
}

Image::Data::~Data()
{
	delete[] data;
}


void Image::load_file(const string &fn)
{
	RefPtr<ImageLoader> loader = ImageLoader::open_file(fn);
	load(*loader);
}

void Image::load_io(IO::Seekable &io)
{
	RefPtr<ImageLoader> loader = ImageLoader::open_io(io);
	load(*loader);
}

void Image::load(ImageLoader &loader)
{
	loader.load(data);
}

} // namespace Graphics
} // namespace Msp
